package filesystem;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**
 * Extends File to provide features missing from the Sun File class.
 *  Also, features a copy and delete (for file and dir) operation.
 * @author Tandeep
 *
 */
public class FileExtended extends File
{
    /**
     * Calls the File constructor
     * @param pathname
     */
    public FileExtended(String pathname)
    {   
        super(pathname);        
    }
    /**
     * Constructor.
     * @param parent parent directory
     * @param fileName the name of the file
     */
    public FileExtended(File parent, String fileName)
    {
        super(parent, fileName);
    }
    /**
     * Constructor.
     * @param parent parent directory
     * @param fileName the name of the file
     */
    public FileExtended(String parent, String fileName)
    {
        super(parent, fileName);
    }
    /**
     * Calls the File constructor
     */
    public FileExtended(FileExtended parent, String file)
    {
        super(parent, file);
    }
    /**
     * Finds all files in a dir. with a particular extension
     * @see File#listFiles(FileFilter)
     */
    public File []listFiles(final String extension)
    {
        FileFilter fileFilter = new FileFilter() {
            public boolean accept(File file) {
                String fileName = file.getName();
                return fileName.toLowerCase().endsWith(extension);
            }
        };
        return listFiles(fileFilter);
    }
    
    /**
     * Returns the name of the file as defined by File.getName() without
     * any attached extension. For filename "image.jpg" will return "image"
     * 
     * Assumption:
     * Extension starts by the last dot(.) in the file name
     */
    public String getNameWithoutExtension()
    {
        String name = getName();
        int index = name.lastIndexOf(".");
        if(index != -1 && index > 1)
            return name.substring(0, index);
        
        return name;
    }
    
    /** 
     * Overrides listFiles in File#listFiles() to only accept files that are
     * not named "CVS". This excludes the cvs files/directories. Useful because
     * our current system has these CVS dirs. which disrupt the system
     * (non-Javadoc)
     * @see java.io.File#listFiles()
     */
    public File[] listFiles()
    {
        return super.listFiles(new FilenameFilter() {

            public boolean accept(File dir, String name)
            {
                if(name.equals("CVS")) return false;
                
                return true;
            }
            
        });
    }
    
    /**
     * Deletes this file/directory. Can delete files and directory even non-empty.
     * 
     * @return True if the file/directory successfuly deleted; false, otherwise
     * @see File#delete()
     */
    public boolean delete()
    {
        if(isFile())
           return super.delete();

        String [] filesAndDirs = list();
        //first delete subdirectories/files
        for(int i = 0 ; filesAndDirs != null && i < filesAndDirs.length; i++)
        {
            FileExtended f = new FileExtended(this, filesAndDirs[i]);
            
            f.delete();
        }
        //then delete this
        return super.delete();
    }
    
    /**
     * Copies a file or directory to dest dir. The file can be 
     * dir or a file.
     * 
     * @param destDir Complete path to the destination directory
     * @param srcFile Complete path to the source file/directory
     * @throws FileNotFoundException Throws FileNotFoundException if srcFile not found
     * @throws IOException Throws IOException if destDir not absolute or does not exist 
     */
    static public void copy(File destDir, File srcFile) throws FileNotFoundException, IOException
    {
    	if(destDir.isAbsolute() == false)
            throw new IOException("Destination dir. " + destDir + " not absolute.");
        //      Create channel on the destination
        if(destDir.exists() == false)
            throw new IOException("Destination Dir. " + destDir + " does not exist");
        
        if(srcFile.isDirectory()) 
        {
        	copyDir(destDir, srcFile);
        	return;
        }
        FileChannel srcChannel = new FileInputStream(srcFile).getChannel();
        FileChannel dstChannel =new FileOutputStream(destDir + "/" + srcFile.getName()).getChannel();
        
        //      Copy file contents from source to destination
        dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
        
        //      Close the channels
        srcChannel.close();
        dstChannel.close();
    }
    
    /**
     * Copies the srcDir to the destDir. Also, copies any 
     * sub-folders and any files under the srcDir.
     * @param destDir
     * @param srcFile
     * @return
     * 
     */
   private static void copyDir(File destDir, File srcDir) throws IOException
    {
        File newFileUnderDest = new File(destDir, srcDir.getName());
        
        newFileUnderDest.mkdir();
       
        Process p = Runtime.getRuntime().exec("xcopy " + srcDir.getAbsolutePath() + "\\* " + newFileUnderDest.getAbsolutePath() + 
        " /a /d /e /c /q /h /r /k /y"); 
        
        try
        {
            p.waitFor();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
    }
   
    /**
     * Given any filename, returns the same file but without the extension(if any)
     * @return New file object where the extension is removed
     */
	public static File removeExtension(File file) {
		
		String  filename = file.getName();
		if(filename.indexOf('.') != -1)
			filename = filename.substring(0, filename.lastIndexOf('.'));
		
		return new File(file.getParent(), filename);
	}
	/**
	 * See {@link FileExtended#getPageNumber() getPageNumber} function for more details
	 */
	public static int getPageNumber(String str) {
		FileExtended temp = new FileExtended(str);
		return temp.getPageNumber();
	}
	  /**
     * Returns the name of the file as defined by File.getName without
     * any attached subpage index or any extension.
     * Assumptions:
     * Page names in format: ce1101-001.html will return ce1101
     */
    private String getNameWithoutExtensionAndSubPage()
    {
        String name = getNameWithoutExtension();
        
        int index = name.lastIndexOf("-");
        if(index != -1 && index > 1)
            return name.substring(0, index);
        
        return name;
    }
	/**
	 * Given any filename, returns any series of digits in that filename.
	 * <pre>
	 * Example: for ce001,  returns 1
	 * for ce0001-002, returns 1
	 * for ce21dd, returns 21
	 * for 21dd, returns 21
	 * for cebdef return -1
	 * </pre>
	 */
	public int getPageNumber() {
		String string = getNameWithoutExtensionAndSubPage();
	
		int i = 0, j = string.length();
		for(i = 0 ; i < string.length(); i++) {
			if(Character.isDigit(string.charAt(i)))
				break;
		}
		for(j = string.length() - 1; j >= i; j--) {
			if(Character.isDigit(string.charAt(j)))
				break;
		}
		String pageNumber = "-1";
		
		if(i < string.length() && i <= j+1)
		{	if(j+1 == i) j = i;
		
			pageNumber = string.substring(i, j+1);
		}
		return Integer.parseInt(pageNumber);
	}
	/**
     * Returns page number prefix. for a filename ce0001, will return "ce". For "abc" returns "abc"
     */
	public String getPageNumberPrefix() {
		int num = getPageNumber();
		String string = getNameWithoutExtensionAndSubPage();

		int i = string.indexOf(Integer.toString(num));
		
		if( i != -1) return string.substring(0, i);
		
		return string;
	}
	/**
	 * Returns the drive name where this file is. Returns null,
	 * if the path for this file is not absolute(in which case there is no drive name)
	 * Example: For filename: C:\Hello\World.txt, returns "C"
	 * <br/> Not sure if this will work on any other system besides Windows
	 */
	public String getHomeDir() {
		if(isAbsolute() == false) return null;
		
		String temp = toString();
		return temp.substring(0, temp.indexOf(':'));
	}
	
	/**
	 * Returns all files in a directory. If the directory contains some sub-directories, then
	 * those files within those folders are also returned including the subfolders.
	 * <p> If a file is passed to this function instead of a directory, only that file is returned. </p>
	 * @param fileName Name of the directory
	 * @return ArrayList of Files
	 */
	public static ArrayList<File> listAllFiles(File dicFile) {
		ArrayList<File> list = new ArrayList<File>();
		
		listAllFilesHelper(dicFile, list);
		
		return list;
		
	}
	
	private static void listAllFilesHelper(File dir, List<File> list) {
		 if (dir.isDirectory()) {
	            String[] children = dir.list();
	            for (int i=0; i<children.length; i++) {
	                listAllFilesHelper(new File(dir, children[i]), list);
	            }
		 }else {
			 list.add(dir);
		 }
		 
		 
	}
	/**
	 * Reads the file specified by the fileName and returns the contets in a vector.
	 * @return Returns the results in a vector. Each line in the file gets saved to an element of the vector. 
	 * So a file with N lines will have N elements in the result vector
	 */
    public static Vector<String> getFileLines(File fileName){
        InputStream is;
        
        try {
            is = new FileInputStream(fileName);
            InputStreamReader inputStreamReader = new InputStreamReader(is);
            BufferedReader bufferReader = new BufferedReader(inputStreamReader);

            String line = (bufferReader.readLine()).trim();
            Vector<String> vLines = new Vector<String>();

            // read in each line
            while (line != null) {
                line = line.trim();
                try{
                    vLines.add(line);
                } catch (NumberFormatException e){
                }
                line = (bufferReader.readLine());
            }
            is.close();
            bufferReader.close();

            return vLines;
        } 
        catch (FileNotFoundException e) {
         } 
        catch (IOException e1) {
        }
    
        return null;
    }

    
    
}


